@@ -5,13 +5,41 @@ from __future__ import annotations
 import functools
 import inspect
 import itertools
-from typing import Any, Callable, Dict, List, Literal, Optional, Sequence, Type, Union
+import sys
+from typing import Any, Callable, Dict, List, Literal, Optional, Sequence, Type, TypeVar, Union
 from unittest import mock as umock
 from unittest.util import safe_repr
 
+if sys.version_info >= (3, 10):
+    from typing import ParamSpec  # pragma: no cover
+else:
+    from typing_extensions import ParamSpec
+
 AnyMock = Union[umock.AsyncMock, umock.MagicMock, umock.PropertyMock]
 AsyncAndSyncMock = Union[umock.AsyncMock, umock.MagicMock]
 
+T = TypeVar("T")
+P = ParamSpec("P")
+
+
+def only_async(fn: Callable[P, T]) -> Callable[P, T]:
+    """Wrapper for async assertions that throws an exception if they are called
+    when the mock is not an AsyncMock.
+    """
+    # pylint: disable=protected-access
+
+    @functools.wraps(fn)
+    def wrapper(*args: P.args, **kwargs: P.kwargs) -> T:
+        self: Assert = args[0]  # type: ignore
+        if not isinstance(self._attr_mock, umock.AsyncMock):
+            raise AttributeError(
+                f"{self._attr_mock.__class__.__name__} does not have '{fn.__name__}' method. "
+                f"You can use 'force_async' parameter to force the mock to be an AsyncMock."
+            )
+        return fn(*args, **kwargs)
+
+    return wrapper
+
 
 class Assert:
     """Assert allows creation of assertions for mocks.
@@ -25,7 +53,6 @@ class Assert:
         self,
         parent: Mock,
         attr_mock: AnyMock,
-        name: str,
         *,
         kind: Literal["spy", "mock"] = "mock",
         patch: Optional[umock._patch[Any]] = None,  # pylint: disable=unsubscriptable-object
@@ -36,8 +63,7 @@ class Assert:
                 "Assert should not be initialized directly. Use mocker function instead."
             )
         self.__parent = parent
-        self.__attr_mock = attr_mock
-        self.__name = name
+        self._attr_mock = attr_mock
         self.__assertions: List[Callable[..., None]] = []
         self.__patch = patch
         self._kind = kind
@@ -78,7 +104,7 @@ class Assert:
         Returns:
             Python unittest mock (`AsyncMock`, `MagicMock`, or `PropertyMock`).
         """
-        return self.__attr_mock
+        return self._attr_mock
 
     def return_value(self, value: Any) -> Assert:
         """Set the value that will be returned when the mocked attribute is
@@ -104,13 +130,13 @@ class Assert:
         Returns:
             Assert instance so that calls can be chained.
         """
-        if isinstance(self.__attr_mock, umock.NonCallableMagicMock) and self.__patch is not None:
+        if isinstance(self._attr_mock, umock.NonCallableMagicMock) and self.__patch is not None:
             # Support mocking module attributes/variables
             self.__patch.stop()
             self.__patch.new = value
             self.__patch.start()
             return self
-        self.__attr_mock.return_value = value
+        self._attr_mock.return_value = value
         return self
 
     def side_effect(self, value: Any) -> Assert:
@@ -169,7 +195,7 @@ class Assert:
         Returns:
             Assert instance so that calls can be chained.
         """
-        self.__attr_mock.side_effect = value
+        self._attr_mock.side_effect = value
         return self
 
     def called_last_with(self, *args: Any, **kwargs: Any) -> Assert:
@@ -194,10 +220,11 @@ class Assert:
             Assert instance so that calls can be chained.
         """
         self.__assertions.append(
-            functools.partial(self.__attr_mock.assert_called_with, *args, **kwargs)
+            functools.partial(self._attr_mock.assert_called_with, *args, **kwargs)
         )
         return self
 
+    @only_async
     def awaited_last_with(self, *args: Any, **kwargs: Any) -> Assert:
         """Assert that the _most recent await_ was with the specified arguments.
 
@@ -220,7 +247,7 @@ class Assert:
             Assert instance so that calls can be chained.
         """
         self.__assertions.append(
-            functools.partial(self.__attr_mock.assert_awaited_with, *args, **kwargs)
+            functools.partial(self._attr_mock.assert_awaited_with, *args, **kwargs)
         )
         return self
 
@@ -262,6 +289,7 @@ class Assert:
         )
         return self
 
+    @only_async
     def match_args_last_await(self, *args: Any, **kwargs: Any) -> Assert:
         """Assert that the _most recent await_ has _at least_ the specified
         arguments.
@@ -323,10 +351,11 @@ class Assert:
             Assert instance so that calls can be chained.
         """
         self.__assertions.append(
-            functools.partial(self.__attr_mock.assert_called_once_with, *args, **kwargs)
+            functools.partial(self._attr_mock.assert_called_once_with, *args, **kwargs)
         )
         return self
 
+    @only_async
     def awaited_once_with(self, *args: Any, **kwargs: Any) -> Assert:
         """Assert that the mock was awaited exactly once with the specified
         arguments.
@@ -350,7 +379,7 @@ class Assert:
             Assert instance so that calls can be chained.
         """
         self.__assertions.append(
-            functools.partial(self.__attr_mock.assert_awaited_once_with, *args, **kwargs)
+            functools.partial(self._attr_mock.assert_awaited_once_with, *args, **kwargs)
         )
         return self
 
@@ -380,10 +409,11 @@ class Assert:
             Assert instance so that calls can be chained.
         """
         self.__assertions.append(
-            functools.partial(self.__attr_mock.assert_any_call, *args, **kwargs)
+            functools.partial(self._attr_mock.assert_any_call, *args, **kwargs)
         )
         return self
 
+    @only_async
     def any_await_with(self, *args: Any, **kwargs: Any) -> Assert:
         """Assert that the mock has been awaited with the specified arguments.
 
@@ -410,7 +440,7 @@ class Assert:
             Assert instance so that calls can be chained.
         """
         self.__assertions.append(
-            functools.partial(self.__attr_mock.assert_any_await, *args, **kwargs)
+            functools.partial(self._attr_mock.assert_any_await, *args, **kwargs)
         )
         return self
 
@@ -436,6 +466,7 @@ class Assert:
         self.__assertions.append(functools.partial(self._assert_all_calls_with, *args, **kwargs))
         return self
 
+    @only_async
     def all_awaits_with(self, *args: Any, **kwargs: Any) -> Assert:
         """Assert that _all_ awaits have the specified arguments.
 
@@ -495,6 +526,7 @@ class Assert:
         )
         return self
 
+    @only_async
     def match_args_any_await(self, *args: Any, **kwargs: Any) -> Assert:
         """Assert that any await has _at least_ the specified arguments.
 
@@ -575,6 +607,7 @@ class Assert:
         )
         return self
 
+    @only_async
     def match_args_all_awaits(self, *args: Any, **kwargs: Any) -> Assert:
         """Assert that _all_ awaits have _at least_ the specified arguments.
 
@@ -649,10 +682,11 @@ class Assert:
             Assert instance so that calls can be chained.
         """
         self.__assertions.append(
-            functools.partial(self.__attr_mock.assert_has_calls, calls, any_order)
+            functools.partial(self._attr_mock.assert_has_calls, calls, any_order)
         )
         return self
 
+    @only_async
     def has_awaits(self, calls: Sequence[umock._Call], any_order: bool = False) -> Assert:
         """Assert that the mock has been awaited with the specified calls.
 
@@ -684,7 +718,7 @@ class Assert:
             Assert instance so that calls can be chained.
         """
         self.__assertions.append(
-            functools.partial(self.__attr_mock.assert_has_awaits, calls, any_order)
+            functools.partial(self._attr_mock.assert_has_awaits, calls, any_order)
         )
         return self
 
@@ -704,9 +738,10 @@ class Assert:
         Returns:
             Assert instance so that calls can be chained.
         """
-        self.__assertions.append(functools.partial(self.__attr_mock.assert_not_called))
+        self.__assertions.append(functools.partial(self._attr_mock.assert_not_called))
         return self
 
+    @only_async
     def not_awaited(self) -> Assert:
         """Assert that the mock was never awaited.
 
@@ -723,7 +758,7 @@ class Assert:
         Returns:
             Assert instance so that calls can be chained.
         """
-        self.__assertions.append(functools.partial(self.__attr_mock.assert_not_awaited))
+        self.__assertions.append(functools.partial(self._attr_mock.assert_not_awaited))
         return self
 
     def called(self) -> Assert:
@@ -743,9 +778,10 @@ class Assert:
         Returns:
             Assert instance so that calls can be chained.
         """
-        self.__assertions.append(functools.partial(self.__attr_mock.assert_called))
+        self.__assertions.append(functools.partial(self._attr_mock.assert_called))
         return self
 
+    @only_async
     def awaited(self) -> Assert:
         """Assert that the mock was awaited at least once.
 
@@ -763,7 +799,7 @@ class Assert:
         Returns:
             Assert instance so that calls can be chained.
         """
-        self.__assertions.append(functools.partial(self.__attr_mock.assert_awaited))
+        self.__assertions.append(functools.partial(self._attr_mock.assert_awaited))
         return self
 
     def called_once(self) -> Assert:
@@ -786,6 +822,7 @@ class Assert:
         self.__assertions.append(functools.partial(self._assert_call_count, 1))
         return self
 
+    @only_async
     def awaited_once(self) -> Assert:
         """Assert that the mock was awaited exactly once.
 
@@ -821,6 +858,7 @@ class Assert:
         self.__assertions.append(functools.partial(self._assert_call_count, 2))
         return self
 
+    @only_async
     def awaited_twice(self) -> Assert:
         """Assert that the mock was awaited exactly twice.
 
@@ -855,6 +893,7 @@ class Assert:
         self.__assertions.append(functools.partial(self._assert_call_count, call_count))
         return self
 
+    @only_async
     def await_count(self, await_count: int) -> Assert:
         """Assert that the mock was awaited the specified number of times.
 
@@ -902,6 +941,7 @@ class Assert:
         self.__assertions.append(functools.partial(self._assert_call_count, call_count, "at least"))
         return self
 
+    @only_async
     def await_count_at_least(self, await_count: int) -> Assert:
         """Assert that the mock was awaited _at least_ the specified number of times.
 
@@ -960,6 +1000,7 @@ class Assert:
         self.__assertions.append(functools.partial(self._assert_call_count, call_count, "at most"))
         return self
 
+    @only_async
     def await_count_at_most(self, await_count: int) -> Assert:
         """Assert that the mock was awaited _at most_ the specified number of times.
 
@@ -1022,50 +1063,52 @@ class Assert:
     def _assert_call_count(
         self, call_count: int, modifier: Optional[Literal["at least", "at most"]] = None
     ) -> None:
-        if modifier is None and self.__attr_mock.call_count == call_count:
+        if modifier is None and self._attr_mock.call_count == call_count:
             return
-        if modifier == "at least" and self.__attr_mock.call_count >= call_count:
+        if modifier == "at least" and self._attr_mock.call_count >= call_count:
             return
-        if modifier == "at most" and self.__attr_mock.call_count <= call_count:
+        if modifier == "at most" and self._attr_mock.call_count <= call_count:
             return
         modifier_str = f"{modifier} " if modifier else ""
+        name = self._attr_mock._mock_name or "mock"  # pylint:disable=protected-access
         msg = (
-            f"Expected '{self.__name}' to have been called {modifier_str}"  # pylint:disable=protected-access
+            f"Expected '{name}' to have been called {modifier_str}"  # pylint:disable=protected-access
             f"{self._format_call_count(call_count)}. "
-            f"Called {self._format_call_count(self.__attr_mock.call_count)}."
-            f"{self.__attr_mock._calls_repr()}"
+            f"Called {self._format_call_count(self._attr_mock.call_count)}."
+            f"{self._attr_mock._calls_repr()}"
         )
         raise AssertionError(msg)
 
     def _assert_await_count(
         self, await_count: int, modifier: Optional[Literal["at least", "at most"]] = None
     ) -> None:
-        if modifier is None and self.__attr_mock.await_count == await_count:
+        if modifier is None and self._attr_mock.await_count == await_count:
             return
-        if modifier == "at least" and self.__attr_mock.await_count >= await_count:
+        if modifier == "at least" and self._attr_mock.await_count >= await_count:
             return
-        if modifier == "at most" and self.__attr_mock.await_count <= await_count:
+        if modifier == "at most" and self._attr_mock.await_count <= await_count:
             return
         modifier_str = f"{modifier} " if modifier else ""
+        name = self._attr_mock._mock_name or "mock"  # pylint:disable=protected-access
         msg = (
-            f"Expected '{self.__name}' to have been awaited {modifier_str}"
+            f"Expected '{name}' to have been awaited {modifier_str}"
             f"{self._format_call_count(await_count)}. "
-            f"Awaited {self._format_call_count(self.__attr_mock.await_count)}."
+            f"Awaited {self._format_call_count(self._attr_mock.await_count)}."
             f"{self._awaits_repr()}"
         )
         raise AssertionError(msg)
 
     def _assert_all_calls_with(self, *args: Any, **kwargs: Any) -> None:
-        if not self._all_args_match(self.__attr_mock.call_args_list, *args, **kwargs):
+        if not self._all_args_match(self._attr_mock.call_args_list, *args, **kwargs):
             msg = (
                 f"All calls have not been made with the given arguments:\n"  # pylint:disable=protected-access
                 f"{self._args_repr(*args, **kwargs)}"
-                f"{self.__attr_mock._calls_repr()}"
+                f"{self._attr_mock._calls_repr()}"
             )
             raise AssertionError(msg)
 
     def _assert_all_awaits_with(self, *args: Any, **kwargs: Any) -> None:
-        if not self._all_args_match(self.__attr_mock.await_args_list, *args, **kwargs):
+        if not self._all_args_match(self._attr_mock.await_args_list, *args, **kwargs):
             msg = (
                 f"All awaits have not been made with the given arguments:\n"
                 f"{self._args_repr(*args, **kwargs)}"
@@ -1084,7 +1127,7 @@ class Assert:
     def _assert_match_call_args(  # pylint: disable=too-many-branches
         self, modifier: Literal["all", "any", "last"], *args: Any, **kwargs: Any
     ) -> None:
-        if not self._assert_match_args(self.__attr_mock.call_args_list, modifier, *args, **kwargs):
+        if not self._assert_match_args(self._attr_mock.call_args_list, modifier, *args, **kwargs):
             if modifier == "last":
                 msg = "Last call does not include arguments"
             elif modifier == "all":
@@ -1094,14 +1137,14 @@ class Assert:
             msg = (
                 f"{msg}:\n"  # pylint:disable=protected-access
                 f"{self._args_repr(*args, **kwargs)}"
-                f"{self.__attr_mock._calls_repr()}"
+                f"{self._attr_mock._calls_repr()}"
             )
             raise AssertionError(msg)
 
     def _assert_match_await_args(  # pylint: disable=too-many-branches
         self, modifier: Literal["all", "any", "last"], *args: Any, **kwargs: Any
     ) -> None:
-        if not self._assert_match_args(self.__attr_mock.await_args_list, modifier, *args, **kwargs):
+        if not self._assert_match_args(self._attr_mock.await_args_list, modifier, *args, **kwargs):
             if modifier == "last":
                 msg = "Last await does not include arguments"
             elif modifier == "all":
@@ -1153,9 +1196,9 @@ class Assert:
 
         Provides similar functionality to `unittest.mock.NonCallableMock._calls_repr`.
         """
-        if not self.__attr_mock.await_args_list:
+        if not self._attr_mock.await_args_list:
             return ""
-        return f"\nAwaits: {safe_repr(self.__attr_mock.await_args_list)}."
+        return f"\nAwaits: {safe_repr(self._attr_mock.await_args_list)}."
 
     @staticmethod
     def _args_repr(*args: Any, **kwargs: Any) -> str:
@@ -1332,9 +1375,7 @@ class Mock:
             return cached
         parsed_name = self.__remove_name_mangling(name)
         original = getattr(self.__target, parsed_name)
-        attr_mock = self.__get_patch_attr_mock(
-            self.__mock, parsed_name, create=True, force_property=False, force_async=False
-        )
+        attr_mock = umock.MagicMock(name=self.__format_mock_name(name))
         parameters = tuple(inspect.signature(original).parameters.keys())
         is_class_method = self.__get_method_type(parsed_name, classmethod)
         is_static_method = self.__get_method_type(parsed_name, staticmethod)
@@ -1350,28 +1391,10 @@ class Mock:
                 args = tuple(list(args)[1:])
             return original(*args, **kwargs)
 
-        async def async_pass_through(*args: Any, **kwargs: Any) -> Any:
-            has_self = len(parameters) > 0 and parameters[0] == "self"
-            skip_first = len(args) > len(parameters) and (is_class_method or is_static_method)
-            mock_args = list(args)
-            if has_self or skip_first:
-                mock_args = mock_args[1:]
-            await attr_mock(*mock_args, **kwargs)
-            if skip_first:
-                args = tuple(list(args)[1:])
-            return await original(*args, **kwargs)
-
-        if isinstance(attr_mock, umock.AsyncMock):
-            patch = umock.patch.object(
-                self.__target, parsed_name, new_callable=lambda: async_pass_through
-            )
-        else:
-            patch = umock.patch.object(
-                self.__target, parsed_name, new_callable=lambda: pass_through
-            )
+        patch = umock.patch.object(self.__target, parsed_name, new=pass_through)
         patch.start()
         self.__object_patches.append(patch)
-        assertion = Assert(self, attr_mock, parsed_name, kind="spy", _internal=True)
+        assertion = Assert(self, attr_mock, kind="spy", _internal=True)
         self.__assertions[name] = assertion
         return assertion
 
@@ -1525,13 +1548,21 @@ class Mock:
         if name in list(set(dir(Mock)) - set(dir(type))):
             raise ValueError(f"Cannot replace Mock internal attribute {name}")
         attr_mock = self.__get_stub_attr_mock(
-            name, create=create, force_property=force_property, force_async=force_async
+            name,
+            create=create,
+            force_property=force_property if not parts else False,
+            force_async=force_async if not parts else False,
         )
-        assertion = Assert(self, attr_mock, name, _internal=True)
+        attr_mock._mock_name = f"Stub.{name}"  # pylint: disable=protected-access
+        assertion = Assert(self, attr_mock, _internal=True)
         if len(parts) > 0:
             # Support for chaining methods
             assertion.return_value(self)
-            assertion = self.mock(".".join(parts))
+            assertion = self.mock(
+                ".".join(parts),
+                force_property=force_property,
+                force_async=force_async,
+            )
         return assertion
 
     def __get_stub_attr_mock(
@@ -1572,19 +1603,28 @@ class Mock:
                 self.__mock(),
                 name,
                 create=create,
-                force_property=force_property,
-                force_async=force_async,
+                force_property=force_property if not parts else False,
+                force_async=force_async if not parts else False,
             )
         else:
             attr_mock = self.__get_patch_attr_mock(
-                self.__mock, name, create=create, force_property=False, force_async=force_async
+                self.__mock,
+                name,
+                create=create,
+                force_property=False,
+                force_async=force_async if not parts else False,
             )
-        assertion = Assert(self, attr_mock, name, _internal=True)
+        assertion = Assert(self, attr_mock, _internal=True)
         if len(parts) > 0:
             # Support for chaining methods
-            stub = Mock(_internal=True)
+            Stub = type("Stub", (Mock,), {})  # Use intermediary class to attach properties
+            stub = Stub(_internal=True)
             assertion.return_value(stub)
-            assertion = stub.mock(".".join(parts))
+            assertion = stub.mock(
+                ".".join(parts),
+                force_property=force_property,
+                force_async=force_async,
+            )
         return assertion
 
     def __get_patch_attr_mock(
@@ -1636,23 +1676,50 @@ class Mock:
             attr_mock = umock.NonCallableMagicMock()
         else:
             new_callable = None
-            if force_property or (original is not None and isinstance(original, property)):
+            if (
+                not parts
+                and force_property
+                or (original is not None and isinstance(original, property))
+            ):
                 new_callable = umock.PropertyMock
-            elif force_async:
+            elif not parts and force_async:
                 new_callable = umock.AsyncMock
             patch = umock.patch.object(
-                self.__target, name, new_callable=new_callable, create=create
+                self.__target,
+                name,
+                new_callable=new_callable,
+                create=create,
+                name=self.__format_mock_name(name),
             )
             attr_mock = patch.start()
             self.__object_patches.append(patch)
-        assertion = Assert(self, attr_mock, name, patch=patch, _internal=True)
+        assertion = Assert(self, attr_mock, patch=patch, _internal=True)
         if len(parts) > 0:
             # Support for chaining methods
-            stub = Mock(_internal=True)
+            Stub = type("Stub", (Mock,), {})  # Use intermediary class to attach properties
+            stub = Stub(_internal=True)
             assertion.return_value(stub)
-            assertion = stub.mock(".".join(parts))
+            assertion = stub.mock(
+                ".".join(parts),
+                force_property=force_property,
+                force_async=force_async,
+            )
         return assertion
 
+    def __format_mock_name(self, name: str) -> str:
+        if self.__target is None:
+            return name
+        target = self.__target
+        if (
+            not inspect.isclass(target)
+            and not inspect.ismodule(target)
+            and hasattr(target, "__class__")
+        ):
+            target = target.__class__
+        if hasattr(target, "__name__"):
+            return f"{target.__name__}.{name}"
+        return name
+
     def _reset(self) -> None:
         while len(self.__object_patches) > 0:
             patch = self.__object_patches.pop()
@@ -1783,14 +1850,12 @@ def mocker(
             this to True. Note that it is usually easier to just use partial
             mocking if you need to patch the class.
         **kwargs: You can give arbitrary keyword arguments to quickly set mocked
-            attributes and properties on the created Mock instance.
+            attributes and properties.
 
     Returns:
         Mock instance.
     """
     mock = State.get_or_create_mock(target, spec=spec, patch_class=patch_class)
     for name, value in kwargs.items():
-        # Create kwargs as properties for stubs without a spec
-        force_property = target is None and spec is None
-        mock.mock(name, force_property=force_property).return_value(value)
+        mock.mock(name, force_property=True).return_value(value)
     return mock
